C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE debugmsg (imsg, dbid, idbg, cstr, ival, rval)

C @(#)Subroutine: debugmsg            Author: R.A. Jansen -- May  8 2013
C @(#)
C @(#)  Purpose: print a numbered informative message when the verbosity
C @(#)    level is raised (>0).  This routine allows the main program to
C @(#)    appear less cluttered.  An arbitrary text string, a real,  and
C @(#)    an integer may also be specified  to be returned  as part of a
C @(#)    message whose exact content cannot be known a priori.
C @(#)
C @(#)  Calling sequence: CALL debugmsg(imsg,dbid,idbg,cstr,ival,rval) ,
C @(#)    where 'ismg' is a integer that uniquely identifies the message
C @(#)    to print, 'dbid' is a 19-character long string identifying the
C @(#)    calling routine, integer 'idbg' is the current verbosity level
C @(#)    and  'cstr'  is character string of arbitrary declared length,
C @(#)    'ival' an arbitrary integer, and 'rval' and arbitrary real.
C @(#)
C @(#)  Dependencies:  several routines from my 'rjgen' library:  cinit,
C @(#)    clen, ifmt
C @(#)
C @(#)  Revision history:
C @(#)    May  8 2013 [RAJ] -- Excised debug messages from MAIN, clinput
C @(#)    May 13 2013 [RAJ] -- Excised debug messages from 'regtyp'; bug
C @(#)                         fixes...
C @(#) -----------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE   'devices.inc'

C --- Declaration of input variables...  
      CHARACTER*(*) cstr, dbid*19
      REAL	rval
      INTEGER	imsg, idbg, ival

C --- External functions...
      INTEGER	clen, ifmt

C --- Other program variables...
      CHARACTER*2048 cmsg, ctmp, cfmt*32
      INTEGER	mlen, ilen, i

C     ------------------------------------------------------------------

C --- Return for default verbosity level...
      IF (idbg.EQ.0) RETURN

C --- Initialize...
      CALL cinit(cmsg)
      CALL cinit(cfmt)

C --- Construct the relevant informative message...
C     -- mkmask MAIN ...
      IF (imsg.EQ.1) THEN
         cmsg = dbid//'calling "clinput"'
      ELSE IF (imsg.EQ.2) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            cmsg = dbid//'read: "'//cstr(1:ilen)//'"'
            WRITE(STDOUT,'(A1)') ' '
         ENDIF
      ELSE IF (imsg.EQ.3) THEN
         i = ifmt(INT(rval))
         WRITE(cfmt,'((A),I1,A1)') '((A),I2,(A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'ityp=',ival,', ilen=',INT(rval)
      ELSE IF (imsg.EQ.4) THEN
         ilen = clen(cstr)
         cmsg = dbid//'reached EOF; closing "'//cstr(1:ilen)//'"'
      ELSE IF (imsg.EQ.5) THEN
         cmsg = 'MKMASK: Finished.'

C     -- mkmask's clinput routine...
      ELSE IF (imsg.EQ.11) THEN
         ilen = clen(cstr)
         cmsg = dbid//'detected command line option=value pair '//
     +          '"'//cstr(1:ilen)//'"'

      ELSE IF (imsg.EQ.12) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'  goodval=',ival
      ELSE IF (imsg.EQ.13) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'  badval=',ival
      ELSE IF (imsg.EQ.14) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'  ncols=',ival
      ELSE IF (imsg.EQ.15) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'  nrows=',ival
      ELSE IF (imsg.EQ.16) THEN
         IF (ABS(rval-NINT(rval)).LT.1e-7) rval = 1.0*NINT(rval)
         WRITE(cfmt,*) rval
         ilen = clen(cfmt)
         cmsg = dbid//'  xoffset='//cfmt(1:ilen)
      ELSE IF (imsg.EQ.17) THEN
         IF (ABS(rval-NINT(rval)).LT.1e-7) rval = 1.0*NINT(rval)
         WRITE(cfmt,*) rval
         ilen = clen(cfmt)
         cmsg = dbid//'  yoffset='//cfmt(1:ilen)
      ELSE IF (imsg.EQ.18) THEN
         ilen = clen(cstr)
         cmsg = dbid//'detected unknown command line input '//
     +          '"'//cstr(1:ilen)//'"'
      ELSE IF (imsg.EQ.19) THEN
         ilen = clen(cstr)
         cmsg = dbid//'read mask file name "'//cstr(1:ilen)//'"'
      ELSE IF (imsg.EQ.20) THEN
         cmsg = dbid//' file name may have been truncated'
      ELSE IF (imsg.EQ.21) THEN
         ilen = clen(cstr)
         cmsg = dbid//'read regions file name "'//cstr(1:ilen)//'"'
      ELSE IF (imsg.EQ.22) THEN
         ilen = clen(cstr)
         cmsg = dbid//'ignored section/extension specifier: mask='//
     +          '"'//cstr(1:ilen)//'"'
      ELSE IF (imsg.EQ.23) THEN
         cmsg = dbid//'mask image exists on input'
      ELSE IF (imsg.EQ.24) THEN
         cmsg = dbid//'mask image does not yet exist on input'
      ELSE IF (imsg.EQ.25) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            cmsg = dbid//' file="'//cstr(1:ilen)//'"'
            WRITE(STDOUT,'(A)') dbid//'calling "rjf_open" (RO)'
         ENDIF
      ELSE IF (imsg.EQ.26) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            CALL cinit(ctmp)
            ctmp = ': "'//cstr(1:ilen)//'"'
            ilen = ilen + 3
            cfmt = '(1x,(A),I3,(A))'
            WRITE(cmsg,cfmt) dbid//'record #',ival,ctmp(1:ilen)
         ENDIF
      ELSE IF (imsg.EQ.27) THEN
         cmsg = dbid//'detected END keyword; lend = T'
      ELSE IF (imsg.EQ.28) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt)dbid//'detected BITPIX keyword; bpp = ',ival
      ELSE IF (imsg.EQ.29) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt)dbid//'detected NAXIS keyword; naxes = ',ival
      ELSE IF (imsg.EQ.30) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt)dbid//'detected NAXIS1 keyword; naxes1 = ',ival
      ELSE IF (imsg.EQ.31) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt)dbid//'detected NAXIS2 keyword; naxes2 = ',ival
      ELSE IF (imsg.EQ.32) THEN
         i = ifmt(ival)
         WRITE(cfmt,'((A),I1,A1)') '((A),I',i,')'
         WRITE(cmsg,cfmt) dbid//'detected NEXTEND keyword; nxtd = ',ival
      ELSE IF (imsg.EQ.33) THEN
         IF (idbg.GT.1) THEN
            i = ifmt(ival)
            WRITE(cfmt,'((A),I1,(A))') '((A),I',i,',A1)'
            WRITE(cmsg,cfmt)dbid//'calling "rjf_close"  (ierr=',ival,')'
         ENDIF
      ELSE IF (imsg.EQ.34) THEN
         ilen = clen(cstr)
         cmsg =dbid//'regions file "'//cstr(1:ilen)//'" exists on input'

C     -- mkmask's regtyp routine...
      ELSE IF (imsg.EQ.51) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            cmsg = dbid//'"'//cstr(1:ilen)//'"'
         ENDIF
      ELSE IF (imsg.EQ.52) THEN
         cmsg = dbid//'detected DS9 regions file format'
      ELSE IF (imsg.EQ.53) THEN
         cmsg = dbid//'ignoring blank descriptor'
      ELSE IF (imsg.EQ.54) THEN
         IF (idbg.GT.1) cmsg = dbid//'treating descriptor as a comment'
      ELSE IF (imsg.EQ.55) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            cmsg = dbid//'testing DS9 descriptor "'//cstr(1:ilen)//'"'
         ENDIF
      ELSE IF (imsg.EQ.56) THEN
         IF (idbg.GT.1) THEN
            ilen = clen(cstr)
            cmsg = dbid//'excised "'//cstr(1:ilen)//'"'
         ENDIF
      ELSE IF (imsg.EQ.57) THEN
         IF (idbg.GT.1) THEN
            i = clen(cstr)
            cmsg =dbid//'analyzing PIXLIST descriptor "'//cstr(1:i)//'"'
         ENDIF
      ELSE IF (imsg.EQ.58) THEN
         IF (idbg.GT.1) THEN
            i = ifmt(ival)
            WRITE(cfmt,'((A),I1,(A))') '((A),I',i,',(A))'
            WRITE(cmsg,cfmt) dbid//' counted ',ival,' fields'
         ENDIF
      ELSE IF (imsg.EQ.59) THEN
         IF (idbg.GT.1) cmsg = dbid//'parsing DS9 descriptor...'
      ELSE IF (imsg.EQ.60) THEN
         ilen = clen(cstr)
         cmsg = dbid//'identified DS9 descriptor "'//cstr(1:ilen)//'()"'
      ELSE IF (imsg.EQ.61) THEN
         cmsg = dbid//'parsing ASCII pixel list descriptor...'
      ELSE IF (imsg.EQ.62) THEN
         ilen = ival
         IF (ilen.GT.0) THEN
            cmsg = dbid//'returned: regstr="'//cstr(1:ilen)//'"'
         ELSE
            cmsg = dbid//'returned: regstr=""'
         ENDIF
         CONTINUE         

C     -- other/unknown routine...
      ELSE
         i = ifmt(imsg)
         WRITE(cfmt,'((A),I1,(A))') '((A),I',i,',(A))'
         WRITE(cmsg,cfmt) dbid//'[debugmsg] requested message ',imsg,
     +          ' not found'
      ENDIF


C --- Write out the requested informative message...
      mlen = clen(cmsg)
      IF (mlen.GT.0) WRITE(STDOUT,'(A)') cmsg(1:mlen)


      RETURN
      END


