C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      CHARACTER*12 FUNCTION degtosex (rcoo, ilon, idig)

C @(#)Subroutine: degtosex           Author: R.A. Jansen -- Sep  2, 2017
C @(#)
C @(#)  Purpose:  Given an input coordinate  'rcoo'  in decimal degrees,
C @(#)    return as the function value its equivalent in sexagesimal no-
C @(#)    tation.  If  'ilon' = 0,  treat the coordinate  as a longitude
C @(#)    [0..360 deg],  dividing it by 15 deg/hr before conversion;  if
C @(#)    'ilon' = 1,  treat the coordinate as a latitude [-90..+90 deg]
C @(#)    and preserving its sign.  Input parameter 'idig' specifies the
C @(#)    number of digits to print after the decimal point [0,1,2]. The
C @(#)    resulting sexagesimal notation will be left-aligned within the
C @(#)    returned character string.
C @(#)
C @(#)  Calling sequence:  scoo = degtosex(rcoo,ilon,idig), where 'rcoo'
C @(#)    is a double-precision real*8,  'ilon'  an integer with a value
C @(#)    of either 0 or 1, and 'idig' an integer with a value of  0, 1,
C @(#)    or 2.  This function has neither input nor exit  error status,
C @(#)    and out-of-range input values are silently ignored  if safe to
C @(#)    do so; otherwise, a warning and empty string is returned.
C @(#)   
C @(#)  Dependencies:      clen
C @(#)
C @(#)  Revision history:
C @(#)    Sep  2, 2017 [RAJ] -- Initial version of this routine.
C @(#)------------------------------------------------------------------

      INTEGER   ilon, idig

      CHARACTER cs*1, ctmp*6
      REAL*8    rcoo, rtmp
      INTEGER   hh, mm, i
      REAL      ss

C --- Initialize the output string...
      degtosex = '            '

C --- Silently modify out-of-range input values when safe to do so;
C     warn and return an empty string otherwise...
      IF (idig.LT.0) idig = 0
      IF (idig.GT.2) idig = 2
      IF (ilon.LT.0) ilon = 0
      IF (ilon.GT.1) ilon = 1
      IF (ilon.EQ.0.AND.rcoo.LT.0.0D0) rcoo = rcoo + 360.0D0
      IF (ilon.EQ.0.AND.rcoo.GT.360.0D0) rcoo = rcoo - 360.0D0
      IF (ilon.EQ.0.AND.(rcoo.LT.0.0D0.OR.rcoo.GT.360.0D0)) THEN
         WRITE(*,'(A)')'[degtosex] ERROR: "rcoo" out of range '//
     +                 '[0.0,360.0]'
         RETURN
      ENDIF
      IF (ilon.EQ.1.AND.(rcoo.LT.-90.0D0.OR.rcoo.GT.90.0D0)) THEN
         WRITE(*,'(A)')'[degtosex] ERROR: "rcoo" out of range '//
     +                 '[-90.0,+90.0]'
         RETURN
      ENDIF

C --- Determine whether to treat the input coordinate as a longitude; if
C     so, divide by 15 deg/hr ...
      IF (ilon.EQ.0) THEN
         rtmp = rcoo/15.
      ELSE
         rtmp = rcoo
      ENDIF

C --- Determine integer hours and minutes; floating point seconds...
      hh = INT(ABS(rtmp))
      mm = INT( 60.0D0*(ABS(rtmp) - DBLE(hh)) )
      ss = REAL( 60.0D0*( 60.0D0*(ABS(rtmp) - DBLE(hh)) - DBLE(mm) ) )
      cs = '+'
      IF (rtmp.LT.0) cs = '-'

C --- Construct the output sexagesimal output string...
      i = 1
      IF (ilon.EQ.1) THEN
         degtosex(i:i) = cs
         i = i + 1
      ENDIF

      ctmp = '      '
      WRITE(ctmp,'(I2.2)') hh
      IF (ctmp(1:1).EQ.' ') ctmp(1:1) = '0'
      degtosex(i:i+1) = ctmp(1:2)
      degtosex(i+2:i+2) = ':'
      i = i + 3

      ctmp = '      '
      WRITE(ctmp,'(I2.2)') mm
      IF (ctmp(1:1).EQ.' ') ctmp(1:1) = '0'
      degtosex(i:i+1) = ctmp(1:2)
      degtosex(i+2:i+2) = ':'
      i = i + 3

      ctmp = '      '
      IF (idig.EQ.0) THEN
         WRITE(ctmp,'(I2.2)') INT(ss)
         IF (ctmp(1:1).EQ.' ') ctmp(1:1) = '0'
         degtosex(i:i+1) = ctmp(1:2)
      ELSE IF (idig.EQ.1) THEN
         WRITE(ctmp,'(F4.1)') ss
         IF (ctmp(1:1).EQ.' ') ctmp(1:1) = '0'
         degtosex(i:i+3) = ctmp(1:4)
      ELSE
         WRITE(ctmp,'(F5.2)') ss
         IF (ctmp(1:1).EQ.' ') ctmp(1:1) = '0'
         degtosex(i:i+4) = ctmp(1:5)
      ENDIF

C     WRITE(*,'(A)')'DEBUG [degtosex]: degtosex="'//degtosex//'"'

      RETURN
      END


