C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      CHARACTER*24 FUNCTION gdate(dfmt,utc)
      
C @(#)Character function: gdate      Author: R.A. Jansen -- Apr 30, 1997
C @(#)
C @(#)  Purpose: Return as the  function value  the current date  and/or
C @(#)    time in a format determined by argument 'dfmt'.   When integer
C @(#)    flag 'utc' equals 0, local date and time are returned; for all
C @(#)    other values, date and time will be UTC.  A FITS Y2K timestamp
C @(#)    ('dfmt=1') is always UTC.   Argument 'dfmt'  has the following
C @(#)    meaning (see manual of "libF77.a" routine 'fdate'):
C @(#)    
C @(#)    dfmt=0  : return full date string  (Ddd Mmm DD hh:mm:ss CCYY);
C @(#)    dfmt=1  : return FITS Y2K date string (CCYY-MM-DDThh:mm:ss);
C @(#)    dfmt=2  : return generic time stamp (DD-Mmm-CCYY hh:mm:ss TZ);
C @(#)    dfmt=3  : return the date (logical) (CCYY-MM-DD);
C @(#)    dfmt=4  : return the date (Europe) (DD-Mmm-CCYY);
C @(#)    dfmt=5  : return the date (US) (Mmm DD, CCYY);
C @(#)    dfmt=6  : return the date (logical) (CCYY/MM/DD);
C @(#)    dfmt=7  : return the date (Europe) (DD/MM/CCYY);
C @(#)    dfmt=8  : return the date (US) (MM/DD/CCYY);
C @(#)    dfmt=9  : return the time to nearest second  (hh:mm:ss TZ);
C @(#)    dfmt=10 : return the year (CCYY);
C @(#)    dfmt=11 : return the abbreviated name of the month (Mmm);
C @(#)    dfmt=12 : return the month number of the year (M or MM);
C @(#)    dfmt=13 : return the abbreviated day of the week (Ddd);
C @(#)    dfmt=14 : return the day number of the month (D or DD);
C @(#)    dfmt=15 : return the fractional day of the year (CCYY.yyyyyyy)
C @(#)    dfmt<0 or dfmt>15 : default to dfmt=0 format.
C @(#)
C @(#)  Calling sequence:
C @(#)    dstr = gdate(dfmt,utc), where 'dstr' and 'gdate' should be de-
C @(#)    clared as CHARACTER*24 and 'dfmt' as an integer.
C @(#)
C @(#)  Note: since 'dfmt' is updated upon exit of this routine, it must
C @(#)    be a variable and not be a constant on input.
C @(#)  Note:  to capture the unlikely case where 'gdate' is called from
C @(#)    within a loop, in which the value of 'dfmt' is incremented and
C @(#)    might be outside the legal range, we save a copy of its value.
C @(#)
C @(#)  Dependencies:     fdate ('libF77.a' /'libg2c.a' / 'libfdate.a'),
C @(#)                    cinit, clen, gutc, creadi ; 'tmzone.inc'
C @(#)
C @(#)  Revision history:
C @(#)    May 20, 2008 [RAJ] -- expanded preamble and documentation.
C @(#)    May 28, 2008 [RAJ] -- added formats:  included FITS timestamp,
C @(#)                          timezone, several new alternate formats.
C @(#)    Jun  6, 2008 [RAJ] -- added argument 'utc' to calling sequence
C @(#)                          and implemented date+time in UTC.
C @(#)    Jun 10, 2008 [RAJ] -- split off UTC computation into seperate
C @(#)                          routine: compute only when requested.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about local time zone...
      INCLUDE   'tmzone.inc'

C --- Declaration of input variables...
      INTEGER   dfmt, utc

C --- Declaration of local variables...
      CHARACTER fdate*24, dstr*24, blank*24
      CHARACTER cyear*4, ccmn*3, cdte*2, cday*3, ctim*8
      CHARACTER chrs*2, cmin*2, csec*2
      CHARACTER cmon(12)*3, cnmon*2, cnmn2*2
      INTEGER   ierr, i, iyear, imon, iday, ihrs, imin, isec
      INTEGER   clen, odfmt
      REAL*8    yfrc, yday

C --- Initialize...
      DATA cmon/'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep',
     +          'Oct','Nov','Dec'/
      CALL cinit(gdate)
      CALL cinit(blank)

C --- Save copy of the value of 'dfmt' with which routine was called... 
      odfmt = dfmt

C --- Test whether dfmt is in the legal range. If not, set dfmt to 0 ...
      IF ((dfmt.LT.0).OR.(dfmt.GT.15)) dfmt = 0

C --- Get the system date+time...
      dstr = fdate()
C      WRITE(*,*)'**debug(gdate): date='//dstr

C --- Padd day numbers <10 to the left with a zero...
      IF (dstr(9:9).EQ.' ') dstr(9:9) = '0'

C --- Parse the date/time information (character strings)...
      cday  = dstr(1:3)
      ccmn  = dstr(5:7)
      cdte  = dstr(9:10)
      ctim  = dstr(12:19)
      chrs  = dstr(12:13)
      cmin  = dstr(15:16)
      csec  = dstr(18:19)
      cyear = dstr(21:24)  
      
C --- Compute equivalent numeric representations (integers)...
      ierr = 0
      CALL creadi(cyear,iyear,ierr)
      IF (ierr.NE.0) THEN
         CALL error('GDATE','reading year from date!')
         iyear = 0
      ENDIF

      imon = 0
      DO i=1,12
         IF (cmon(i).EQ.ccmn) imon = i
      ENDDO
      IF (imon.LT.10) THEN
         WRITE(cnmon,'(I1,A1)') imon, ' '
         WRITE(cnmn2,'(A1,I1)') '0', imon
      ELSE
         WRITE(cnmon,'(I2)') imon
         cnmn2 = cnmon
      ENDIF
      IF (imon.EQ.0) THEN
         CALL error('GDATE','reading month from date!')
         imon = 1
      ENDIF

      ierr = 0
      CALL creadi(cdte,iday,ierr)
      IF (ierr.NE.0) THEN
         CALL error('GDATE','reading day from date!')
         iday = 1
      ENDIF
      
      ierr = 0
      CALL creadi(chrs,ihrs,ierr)
      IF (ierr.NE.0) THEN
         CALL error('GDATE','reading hours from date!')
         ihrs = 0
      ENDIF
      
      ierr = 0 
      CALL creadi(cmin,imin,ierr)
      IF (ierr.NE.0) THEN
         CALL error('GDATE','reading minutes from date!')
         imin = 0
      ENDIF

      ierr = 0
      CALL creadi(csec,isec,ierr)
      IF (ierr.NE.0) THEN
         CALL error('GDATE','reading seconds from date!')
         isec = 0
      ENDIF

      
C --- Decide whether to return local time or UTC...
      IF (utc.NE.0.OR.dfmt.EQ.1) THEN
         ierr = 0
         CALL gutc(iyear,imon,iday,ihrs,imin,isec,ierr)
         IF (ierr.NE.0)
     +      CALL error('GDATE','converting local time to UTC!')
      
         WRITE(ctim,'(I2,A1,I2,A1,I2)') ihrs, ':', imin, ':', isec
         IF (ctim(1:1).EQ.' ') ctim(1:1) = '0'
         IF (ctim(4:4).EQ.' ') ctim(4:4) = '0'
         IF (ctim(7:7).EQ.' ') ctim(7:7) = '0'
      ENDIF
         
C      WRITE(*,*)'**debug(gdate): iyear=',iyear,', imon=',imon,', iday=',
C     +          iday,', ihrs=',ihrs,', imin=',imin,', isec=',isec

      
C --- Return the date and/or time in the desired format...
      IF (dfmt.EQ.0) THEN
         gdate = dstr
      ELSE IF (dfmt.EQ.1) THEN
         WRITE(dstr,'(I4,A1,I2,A1,I2,A1,A8)') iyear, '-', imon, '-',
     +         iday, 'T', ctim
         IF (dstr(6:6).EQ.' ') dstr(6:6) = '0'
         IF (dstr(9:9).EQ.' ') dstr(9:9) = '0'
         gdate = dstr
      ELSE IF (dfmt.EQ.2) THEN
         dstr = cdte//'-'//ccmn//'-'//cyear//' '//ctim//' '//TIMEZONE
         gdate = dstr
      ELSE IF (dfmt.EQ.3) THEN
         gdate = cyear//'-'//cnmn2//'-'//cdte//blank(1:(24-12))
      ELSE IF (dfmt.EQ.4) THEN
         gdate = cdte//'-'//ccmn//'-'//cyear//blank(1:(24-11))
      ELSE IF (dfmt.EQ.5) THEN
         gdate = ccmn//' '//cdte//', '//cyear//blank(1:(24-12))
      ELSE IF (dfmt.EQ.6) THEN
         gdate = cyear//'/'//cnmn2//'/'//cdte//blank(1:(24-10))   
      ELSE IF (dfmt.EQ.7) THEN
         gdate = cdte//'/'//cnmn2//'/'//cyear//blank(1:(24-10))
      ELSE IF (dfmt.EQ.8) THEN
         gdate = cnmn2//'/'//cdte//'/'//cyear//blank(1:(24-10))
      ELSE IF (dfmt.EQ.9) THEN
         gdate = ctim//' '//TIMEZONE//blank(1:(24-8-clen(TIMEZONE)-1))  
      ELSE IF (dfmt.EQ.10) THEN
         gdate = cyear//blank(1:(24-4))
      ELSE IF (dfmt.EQ.11) THEN
         gdate = ccmn//blank(1:(24-3))
      ELSE IF (dfmt.EQ.12) THEN
         gdate = cnmon//blank(1:(24-clen(cnmon)))
      ELSE IF (dfmt.EQ.13) THEN
         gdate = cday//blank(1:(24-3))
      ELSE IF (dfmt.EQ.14) THEN
         gdate = cdte//blank(1:(24-2))
      ELSE
         yday = 365.00D0
         IF (MOD(iyear,4).EQ.0) yday = 366.00D0
         IF (MOD(iyear,100).EQ.0) yday = 365.00D0
         IF (MOD(iyear,400).EQ.0) yday = 366.00D0
         IF (MOD(iyear,1000).EQ.0) yday = 365.00D0
         yfrc = 0.00D0
         IF (imon.GT.1) yfrc = yfrc + 31.0D0
         IF (imon.GT.2) THEN
            IF (yday.GT.365.0D0) THEN
               yfrc = yfrc + 29.0D0
            ELSE
               yfrc = yfrc + 28.0D0
            ENDIF
         ENDIF
         IF (imon.GT.3)  yfrc = yfrc + 31.0D0
         IF (imon.GT.4)  yfrc = yfrc + 30.0D0
         IF (imon.GT.5)  yfrc = yfrc + 31.0D0
         IF (imon.GT.6)  yfrc = yfrc + 30.0D0
         IF (imon.GT.7)  yfrc = yfrc + 31.0D0
         IF (imon.GT.8)  yfrc = yfrc + 31.0D0
         IF (imon.GT.9)  yfrc = yfrc + 30.0D0
         IF (imon.GT.10) yfrc = yfrc + 31.0D0
         IF (imon.GT.11) yfrc = yfrc + 30.0D0
         yfrc = yfrc + DBLE(iday-1)
         yfrc = yfrc + DBLE((isec/60.+imin)/60. +ihrs)/24.0D0
         yfrc = DBLE(iyear) + yfrc/yday
         WRITE(dstr,'(F13.8)') yfrc
         gdate = dstr
      ENDIF
C      WRITE(*,*)'**debug(gdate): dstr='//gdate
         

C --- Reset dfmt to the original value
      dfmt = odfmt
      
      RETURN
      END
         

