C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE readline (iunit, cline, ilen, eof, ierr)

C @(#)Subroutine: readline           Author: R.A. Jansen -- Mar 28, 2013
C @(#)
C @(#)  Purpose: Try to read the next line from an already open file and
C @(#)    when successful return it to the calling routine.  Also return
C @(#)    the length of that line, and a status flag to indicate whether
C @(#)    the end of the file was reached.
C @(#)
C @(#)  Calling sequence: CALL readline(iunit,cline,ilen,eof,idbg,ierr),
C @(#)    where integer 'iunit' is the logical device unit from which to
C @(#)    read  and which corresponds to  the already open file. 'cline'
C @(#)    is a character string of arbitrary declared length and integer
C @(#)    'ilen' is the actual length of the line as read.  Boolean flag
C @(#)    'eof' will be returned .TRUE. when the end-of-file is reached.
C @(#)    Integer 'ierr' will be non-zero upon exit, if any error condi-
C @(#)    tion is detected.
C @(#)
C @(#)  Dependencies:  clen, cinit, error; 'devices.inc'
C @(#)
C @(#)  Revision history:
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Declaration of input/output variables; external function...
      CHARACTER*(*) cline
      INTEGER	iunit, ilen, ierr, clen
      LOGICAL	eof


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize output variables...
      CALL cinit(cline)
      ilen = -1
      eof  = .FALSE.

C --- Try to read the next line from the currently open file...
      READ (iunit,'(A)',IOSTAT=ierr) cline
      IF (ierr.EQ.0) THEN
C        Succesfully read a line from this file...
         ilen = clen(cline)
      ELSE IF (ierr.LT.0) THEN
C        Successfully read the last line in this file...
         ierr = 0
         eof  = .TRUE.
         ilen = clen(cline)
      ELSE
C        Failed to read a line from this file, or unexpected EOF...
         CALL error('READLINE','read failed or unexpected EOF!')
         ierr = 1
      ENDIF

      RETURN
      END


