C @(#)NOTE1: the following code implements trigonometric functions  that
C @(#)      became intrinsic functions in F85/90 and some F77 extensions
C @(#)      but are not implemented in compilers  that adhere  to strict
C @(#)      F77 syntax.  Despite this,  such compilers may recognize the
C @(#)      function names as reserved tokens and complain when routines
C @(#)      try to invoke them. In such cases, make sure you declare not
C @(#)      only the function's variable type in the calling routine but
C @(#)      also announce the routine as an EXTERNAL function.
C @(#)NOTE2: while the function implementations in 'trigd.f'  use REAL*8
C @(#)      variables internally, both their input and returned function
C @(#)      value are merely REALs. This means that loss of precision is
C @(#)      possible.  Programmer beware.  R.A. Jansen -- Apr 23 2013.

C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION DTR(degrees)

C @(#)Function: DTR                   Author: R.A. Jansen -- Nov 25 1994
C @(#)
C @(#)  Purpose:  Return as the function value the angle in radians that
C @(#)     corresponds to the given angle in degrees.
C @(#)
C @(#)  Calling sequence: rrad = DTR(rdeg) , where 'rdeg' and 'rrad' are
C @(#)    floating point real values.
C @(#)
C @(#)  Dependencies:     (none)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- expanded documentation.
C @(#)------------------------------------------------------------------

      REAL      degrees
      REAL*8    PI
      PARAMETER (PI=3.14159265358979D0)

      DTR = REAL(DBLE(degrees)*PI/1.80D2)

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION RTD(radians)

C @(#)Function: RTD                   Author: R.A. Jansen -- Nov 25 1994       
C @(#)
C @(#)  Purpose:  Return as the function value the angle in degrees that
C @(#)     corresponds to the given angle in radians.
C @(#)
C @(#)  Calling sequence: rdeg = RTD(rrad) , where 'rrad' and 'rdeg' are
C @(#)    floating point real values.
C @(#)
C @(#)  Dependencies:     (none)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- expanded documentation.
C @(#)------------------------------------------------------------------

      REAL      radians
      REAL*8    PI
      PARAMETER (PI=3.14159265358979D0)

      RTD = REAL(DBLE(radians)*1.80D2/PI)

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION SIND(degrees)

C @(#)Function: SIND                  Author: R.A. Jansen -- Apr 23 2013
C @(#)
C @(#)  Purpose:  Return  as the function value  the sine  of the  given
C @(#)    angle in degrees.
C @(#)
C @(#)  Calling sequence: rsin = SIND(rdeg), where 'rsin' and 'rdeg' are
C @(#)    floating point real values.
C @(#)
C @(#)  Dependencies:     DSIN (intrinsic double precision function)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- initial version.
C @(#)------------------------------------------------------------------

      REAL      degrees
      REAL*8    PI, dtr
      PARAMETER (PI=3.14159265358979D0)

      dtr  = DBLE(degrees)*PI/1.80D2
      SIND = REAL(DSIN(dtr))

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION COSD(degrees)

C @(#)Function: COSD                  Author: R.A. Jansen -- Apr 23 2013
C @(#)
C @(#)  Purpose:  Return as the function value  the cosine  of the given
C @(#)    angle in degrees.
C @(#)
C @(#)  Calling sequence: rcos = COSD(rdeg), where 'rcos' and 'rdeg' are
C @(#)    floating point real values.
C @(#)
C @(#)  Dependencies:     DCOS (intrinsic double precision function)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- initial version.
C @(#)------------------------------------------------------------------

      REAL      degrees
      REAL*8    PI, dtr
      PARAMETER (PI=3.14159265358979D0)

      dtr  = DBLE(degrees)*PI/1.80D2
      COSD = REAL(DCOS(dtr))

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION TAND(degrees)

C @(#)Function: TAND                  Author: R.A. Jansen -- Apr 23 2013
C @(#)
C @(#)  Purpose: Return as the function value  the tangent  of the given
C @(#)    angle in degrees.
C @(#)
C @(#)  Calling sequence: rtan = TAND(rdeg), where 'rtan' and 'rdeg' are
C @(#)    floating point real values.
C @(#)
C @(#)  Dependencies:     DSIN, DCOS (intrinsic dble functions)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- initial version.
C @(#)------------------------------------------------------------------

      REAL      degrees
      REAL*8    PI, dtr
      PARAMETER (PI=3.14159265358979D0)

      dtr  = DBLE(degrees)*PI/1.80D2
      TAND = REAL(DSIN(dtr)/DCOS(dtr))

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION ATAND(ratio)

C @(#)Function: ATAND                 Author: R.A. Jansen -- Apr 23 2013
C @(#)
C @(#)  Purpose:  Return as the function value in degrees the arctangent
C @(#)    of the given argument (ratio).
C @(#)
C @(#)  Calling sequence:  rdeg = ATAND(rdiv)  , where 'rdeg' and 'rdiv'
C @(#)    are floating point real values.
C @(#)
C @(#)  Dependencies:     DATAN (intrinsic double precision function)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- initial version.
C @(#)------------------------------------------------------------------

      REAL      ratio
      REAL*8    PI
      PARAMETER (PI=3.14159265358979D0)

      ATAND = REAL(DATAN(DBLE(ratio))*1.80D2/PI)

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      REAL FUNCTION ATAN2D(ry,rx)

C @(#)Function: ATAN2D                Author: R.A. Jansen -- Apr 23 2013
C @(#)
C @(#)  Purpose:  Return as the function value in degrees the arctangent
C @(#)    of the ratio (ry/rx), given as the two arguments ry and rx.
C @(#)
C @(#)  Calling sequence: rtan = ATAN2D(y,x), where 'rtan', 'y', and 'x'
C @(#)    are all floating point real values.
C @(#)
C @(#)  Dependencies:     DATAN2 (intrinsic double precision function)
C @(#)
C @(#)  Revision history: Apr 23, 2013 [RAJ] -- initial version.
C @(#)                    Sep  4, 2017 [RAJ] -- fixed  test of  equality
C @(#)                                          rx,ry with integer 0->0.
C @(#)------------------------------------------------------------------

      REAL      ry, rx
      REAL*8    PI
      PARAMETER (PI=3.14159265358979D0)

      IF (rx.GT.0.) THEN
         ATAN2D = REAL(DATAN(DBLE(ry)/DBLE(rx))*1.80D2/PI)
      ELSE IF (rx.LT.0..AND.ry.GE.0.) THEN
         ATAN2D = REAL((DATAN(DBLE(ry)/DBLE(rx))+PI)*1.80D2/PI)
      ELSE IF (rx.LT.0..AND.ry.LT.0.) THEN
         ATAN2D = REAL((DATAN(DBLE(ry)/DBLE(rx))-PI)*1.80D2/PI)
      ELSE IF (ABS(rx).LT.1.0e-17.AND.ry.GT.0.) THEN
         ATAN2D = 90.0
      ELSE IF (ABS(rx).LT.1.0e-17.AND.ry.LT.0.) THEN
         ATAN2D = -90.0
      ELSE
C        Formally, ATAN2 is undefined for y=x=0 ; set to +90 deg for
C        consistency with SuperMongo implementation?
         ATAN2D = 90.0
      ENDIF

      RETURN
      END


