C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE wsizstr(styp,naxes,axlen,sizstr,slen,ierr)

C @(#)Subroutine: wsizstr            Author: R.A. Jansen -- Oct 25, 2000
C @(#)
C @(#)  Purpose: given the number of axes in a data array and the length
C @(#)    (number of elements) of each axis and given a format specifier
C @(#)    return a data array dimension text string as:
C @(#)
C @(#)    styp=0 (default) :  "[axlen1,axlen2,...,axlenN]"   (N=naxes)
C @(#)    styp=1           :  "axlen1*axlen2*...*axlenN"
C @(#)    styp=2           :  "[1:axlen1,1:axlen2,...,1:axlenN]"
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL wsizstr (styp, naxes, axlen, sizstr, slen, ierr)  , where
C @(#)    'styp' is an integer with valid values 0 and 1, 'naxes' is the
C @(#)    integer number of axes  in the data array,  'axlen' an integer
C @(#)    array of  'naxes'  axis lengths  (number of pixels  along each
C @(#)    image axis).  Character string 'sizstr' has arbitrary declared
C @(#)    length and will return the formatted text string,  and integer
C @(#)    'slen'  will return the actual (as opposed to declared) length
C @(#)    of 'sizstr'.  Integer flag  'ierr'  will be non-zero upon exit
C @(#)    should any error condition be triggered.
C @(#)
C @(#)  Dependencies:     cinit, clen
C @(#)
C @(#)  Revision history:
C @(#)    May 24, 2008 [RAJ] -- expanded documentation.
C @(#)    Aug  3, 2008 [RAJ] -- changed explicit to variable formatting;
C @(#)                          moved 'styp' to be the first variable in
C @(#)                          the calling sequence.
C @(#)    Aug 31, 2008 [RAJ] -- generalized routine  to accommodate data
C @(#)                          arrays of arbitrary dimension.
C @(#)    Mar 20, 2013 [RAJ] -- added 'slen' to calling sequence, retur-
C @(#)                          ning the actual length of 'sizstr' (aids
C @(#)                          calling routine); also implemented a new
C @(#)                          'styp'=2;  updated documentation;  fixed
C @(#)                          potential bug of LOG(axlen(j)=0).
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER*(*) sizstr
      INTEGER	styp, naxes, axlen(*), slen, ierr

C --- Declaration of local variables...
      CHARACTER	ctmp*80, cch*1, fmt*8
      INTEGER	i, j, clen


C --- Initialize...
      CALL cinit(sizstr)

C --- Return for non-zero input error status, trivial or bad input...
      IF (ierr.NE.0) RETURN
      IF (styp.LT.0.OR.styp.GT.2) styp = 0
      IF (naxes.EQ.0) THEN
         sizstr = '[0]'
         slen   = 3
         RETURN
      ENDIF
      DO j=1,naxes
         IF (axlen(j).LE.0) ierr = 1
         IF (ierr.NE.0) RETURN
      ENDDO

C --- Check whether 'sizstr' is long enough to hold the image size...
      i = 2
      IF (styp.EQ.1) i = 0
      IF (styp.EQ.2) i = 2*naxes + 2
      DO j=1,naxes
         i = i + INT(LOG10(FLOAT(axlen(j)))) + 1
      ENDDO
      IF (i.GT.LEN(sizstr)) ierr = 2
      IF (ierr.NE.0) RETURN

C --- Format will be either [NCOL,NROW,..] or 'NCOLxNROWx..' ...
      IF (styp.EQ.1) THEN
         cch = 'x'
      ELSE
         cch = ','
         IF (styp.EQ.0) sizstr = '['
         IF (styp.EQ.2) sizstr = '[1:'
      ENDIF

C --- Write the image size string...
      DO j=1,naxes
         i = INT(LOG10(FLOAT(axlen(j))))+1
         IF (i.LT.10) THEN
            WRITE(fmt,'((A),I1,(A))') '(I',i,')'
            WRITE(ctmp,fmt) axlen(j)
         ELSE
            WRITE(fmt,'((A),I2,(A))') '(I',i,')'
            WRITE(ctmp,fmt) axlen(j)
         ENDIF
         sizstr = sizstr(1:clen(sizstr))//ctmp(1:clen(ctmp))
         IF (j.LT.naxes) THEN
            sizstr = sizstr(1:clen(sizstr))//cch
            IF (styp.EQ.2) sizstr = sizstr(1:clen(sizstr))//'1:'
         ENDIF
      ENDDO

C --- Return the result... 
      IF (styp.EQ.0.OR.styp.EQ.2) sizstr = sizstr(1:clen(sizstr))//']'
      slen = clen(sizstr)

      RETURN
      END


